; This file specifies the import forwarding for python3.dll
; It is used when building python3dll.vcxproj
LIBRARY	"python3"
EXPORTS
  PyArg_Parse=python35.PyArg_Parse
  PyArg_ParseTuple=python35.PyArg_ParseTuple
  PyArg_ParseTupleAndKeywords=python35.PyArg_ParseTupleAndKeywords
  PyArg_UnpackTuple=python35.PyArg_UnpackTuple
  PyArg_VaParse=python35.PyArg_VaParse
  PyArg_VaParseTupleAndKeywords=python35.PyArg_VaParseTupleAndKeywords
  PyArg_ValidateKeywordArguments=python35.PyArg_ValidateKeywordArguments
  PyBaseObject_Type=python35.PyBaseObject_Type DATA
  PyBool_FromLong=python35.PyBool_FromLong
  PyBool_Type=python35.PyBool_Type DATA
  PyByteArrayIter_Type=python35.PyByteArrayIter_Type DATA
  PyByteArray_AsString=python35.PyByteArray_AsString
  PyByteArray_Concat=python35.PyByteArray_Concat
  PyByteArray_FromObject=python35.PyByteArray_FromObject
  PyByteArray_FromStringAndSize=python35.PyByteArray_FromStringAndSize
  PyByteArray_Resize=python35.PyByteArray_Resize
  PyByteArray_Size=python35.PyByteArray_Size
  PyByteArray_Type=python35.PyByteArray_Type DATA
  PyBytesIter_Type=python35.PyBytesIter_Type DATA
  PyBytes_AsString=python35.PyBytes_AsString
  PyBytes_AsStringAndSize=python35.PyBytes_AsStringAndSize
  PyBytes_Concat=python35.PyBytes_Concat
  PyBytes_ConcatAndDel=python35.PyBytes_ConcatAndDel
  PyBytes_DecodeEscape=python35.PyBytes_DecodeEscape
  PyBytes_FromFormat=python35.PyBytes_FromFormat
  PyBytes_FromFormatV=python35.PyBytes_FromFormatV
  PyBytes_FromObject=python35.PyBytes_FromObject
  PyBytes_FromString=python35.PyBytes_FromString
  PyBytes_FromStringAndSize=python35.PyBytes_FromStringAndSize
  PyBytes_Repr=python35.PyBytes_Repr
  PyBytes_Size=python35.PyBytes_Size
  PyBytes_Type=python35.PyBytes_Type DATA
  PyCFunction_Call=python35.PyCFunction_Call
  PyCFunction_ClearFreeList=python35.PyCFunction_ClearFreeList
  PyCFunction_GetFlags=python35.PyCFunction_GetFlags
  PyCFunction_GetFunction=python35.PyCFunction_GetFunction
  PyCFunction_GetSelf=python35.PyCFunction_GetSelf
  PyCFunction_New=python35.PyCFunction_New
  PyCFunction_NewEx=python35.PyCFunction_NewEx
  PyCFunction_Type=python35.PyCFunction_Type DATA
  PyCallIter_New=python35.PyCallIter_New
  PyCallIter_Type=python35.PyCallIter_Type DATA
  PyCallable_Check=python35.PyCallable_Check
  PyCapsule_GetContext=python35.PyCapsule_GetContext
  PyCapsule_GetDestructor=python35.PyCapsule_GetDestructor
  PyCapsule_GetName=python35.PyCapsule_GetName
  PyCapsule_GetPointer=python35.PyCapsule_GetPointer
  PyCapsule_Import=python35.PyCapsule_Import
  PyCapsule_IsValid=python35.PyCapsule_IsValid
  PyCapsule_New=python35.PyCapsule_New
  PyCapsule_SetContext=python35.PyCapsule_SetContext
  PyCapsule_SetDestructor=python35.PyCapsule_SetDestructor
  PyCapsule_SetName=python35.PyCapsule_SetName
  PyCapsule_SetPointer=python35.PyCapsule_SetPointer
  PyCapsule_Type=python35.PyCapsule_Type DATA
  PyClassMethodDescr_Type=python35.PyClassMethodDescr_Type DATA
  PyCodec_BackslashReplaceErrors=python35.PyCodec_BackslashReplaceErrors
  PyCodec_Decode=python35.PyCodec_Decode
  PyCodec_Decoder=python35.PyCodec_Decoder
  PyCodec_Encode=python35.PyCodec_Encode
  PyCodec_Encoder=python35.PyCodec_Encoder
  PyCodec_IgnoreErrors=python35.PyCodec_IgnoreErrors
  PyCodec_IncrementalDecoder=python35.PyCodec_IncrementalDecoder
  PyCodec_IncrementalEncoder=python35.PyCodec_IncrementalEncoder
  PyCodec_KnownEncoding=python35.PyCodec_KnownEncoding
  PyCodec_LookupError=python35.PyCodec_LookupError
  PyCodec_Register=python35.PyCodec_Register
  PyCodec_RegisterError=python35.PyCodec_RegisterError
  PyCodec_ReplaceErrors=python35.PyCodec_ReplaceErrors
  PyCodec_StreamReader=python35.PyCodec_StreamReader
  PyCodec_StreamWriter=python35.PyCodec_StreamWriter
  PyCodec_StrictErrors=python35.PyCodec_StrictErrors
  PyCodec_XMLCharRefReplaceErrors=python35.PyCodec_XMLCharRefReplaceErrors
  PyComplex_FromDoubles=python35.PyComplex_FromDoubles
  PyComplex_ImagAsDouble=python35.PyComplex_ImagAsDouble
  PyComplex_RealAsDouble=python35.PyComplex_RealAsDouble
  PyComplex_Type=python35.PyComplex_Type DATA
  PyDescr_NewClassMethod=python35.PyDescr_NewClassMethod
  PyDescr_NewGetSet=python35.PyDescr_NewGetSet
  PyDescr_NewMember=python35.PyDescr_NewMember
  PyDescr_NewMethod=python35.PyDescr_NewMethod
  PyDictItems_Type=python35.PyDictItems_Type DATA
  PyDictIterItem_Type=python35.PyDictIterItem_Type DATA
  PyDictIterKey_Type=python35.PyDictIterKey_Type DATA
  PyDictIterValue_Type=python35.PyDictIterValue_Type DATA
  PyDictKeys_Type=python35.PyDictKeys_Type DATA
  PyDictProxy_New=python35.PyDictProxy_New
  PyDictProxy_Type=python35.PyDictProxy_Type DATA
  PyDictValues_Type=python35.PyDictValues_Type DATA
  PyDict_Clear=python35.PyDict_Clear
  PyDict_Contains=python35.PyDict_Contains
  PyDict_Copy=python35.PyDict_Copy
  PyDict_DelItem=python35.PyDict_DelItem
  PyDict_DelItemString=python35.PyDict_DelItemString
  PyDict_GetItem=python35.PyDict_GetItem
  PyDict_GetItemString=python35.PyDict_GetItemString
  PyDict_GetItemWithError=python35.PyDict_GetItemWithError
  PyDict_Items=python35.PyDict_Items
  PyDict_Keys=python35.PyDict_Keys
  PyDict_Merge=python35.PyDict_Merge
  PyDict_MergeFromSeq2=python35.PyDict_MergeFromSeq2
  PyDict_New=python35.PyDict_New
  PyDict_Next=python35.PyDict_Next
  PyDict_SetItem=python35.PyDict_SetItem
  PyDict_SetItemString=python35.PyDict_SetItemString
  PyDict_Size=python35.PyDict_Size
  PyDict_Type=python35.PyDict_Type DATA
  PyDict_Update=python35.PyDict_Update
  PyDict_Values=python35.PyDict_Values
  PyEllipsis_Type=python35.PyEllipsis_Type DATA
  PyEnum_Type=python35.PyEnum_Type DATA
  PyErr_BadArgument=python35.PyErr_BadArgument
  PyErr_BadInternalCall=python35.PyErr_BadInternalCall
  PyErr_CheckSignals=python35.PyErr_CheckSignals
  PyErr_Clear=python35.PyErr_Clear
  PyErr_Display=python35.PyErr_Display
  PyErr_ExceptionMatches=python35.PyErr_ExceptionMatches
  PyErr_Fetch=python35.PyErr_Fetch
  PyErr_Format=python35.PyErr_Format
  PyErr_FormatV=python35.PyErr_FormatV
  PyErr_GivenExceptionMatches=python35.PyErr_GivenExceptionMatches
  PyErr_NewException=python35.PyErr_NewException
  PyErr_NewExceptionWithDoc=python35.PyErr_NewExceptionWithDoc
  PyErr_NoMemory=python35.PyErr_NoMemory
  PyErr_NormalizeException=python35.PyErr_NormalizeException
  PyErr_Occurred=python35.PyErr_Occurred
  PyErr_Print=python35.PyErr_Print
  PyErr_PrintEx=python35.PyErr_PrintEx
  PyErr_ProgramText=python35.PyErr_ProgramText
  PyErr_Restore=python35.PyErr_Restore
  PyErr_SetFromErrno=python35.PyErr_SetFromErrno
  PyErr_SetFromErrnoWithFilename=python35.PyErr_SetFromErrnoWithFilename
  PyErr_SetFromErrnoWithFilenameObject=python35.PyErr_SetFromErrnoWithFilenameObject
  PyErr_SetInterrupt=python35.PyErr_SetInterrupt
  PyErr_SetNone=python35.PyErr_SetNone
  PyErr_SetObject=python35.PyErr_SetObject
  PyErr_SetString=python35.PyErr_SetString
  PyErr_SyntaxLocation=python35.PyErr_SyntaxLocation
  PyErr_WarnEx=python35.PyErr_WarnEx
  PyErr_WarnExplicit=python35.PyErr_WarnExplicit
  PyErr_WarnFormat=python35.PyErr_WarnFormat
  PyErr_WriteUnraisable=python35.PyErr_WriteUnraisable
  PyEval_AcquireLock=python35.PyEval_AcquireLock
  PyEval_AcquireThread=python35.PyEval_AcquireThread
  PyEval_CallFunction=python35.PyEval_CallFunction
  PyEval_CallMethod=python35.PyEval_CallMethod
  PyEval_CallObjectWithKeywords=python35.PyEval_CallObjectWithKeywords
  PyEval_EvalCode=python35.PyEval_EvalCode
  PyEval_EvalCodeEx=python35.PyEval_EvalCodeEx
  PyEval_EvalFrame=python35.PyEval_EvalFrame
  PyEval_EvalFrameEx=python35.PyEval_EvalFrameEx
  PyEval_GetBuiltins=python35.PyEval_GetBuiltins
  PyEval_GetCallStats=python35.PyEval_GetCallStats
  PyEval_GetFrame=python35.PyEval_GetFrame
  PyEval_GetFuncDesc=python35.PyEval_GetFuncDesc
  PyEval_GetFuncName=python35.PyEval_GetFuncName
  PyEval_GetGlobals=python35.PyEval_GetGlobals
  PyEval_GetLocals=python35.PyEval_GetLocals
  PyEval_InitThreads=python35.PyEval_InitThreads
  PyEval_ReInitThreads=python35.PyEval_ReInitThreads
  PyEval_ReleaseLock=python35.PyEval_ReleaseLock
  PyEval_ReleaseThread=python35.PyEval_ReleaseThread
  PyEval_RestoreThread=python35.PyEval_RestoreThread
  PyEval_SaveThread=python35.PyEval_SaveThread
  PyEval_ThreadsInitialized=python35.PyEval_ThreadsInitialized
  PyExc_ArithmeticError=python35.PyExc_ArithmeticError DATA
  PyExc_AssertionError=python35.PyExc_AssertionError DATA
  PyExc_AttributeError=python35.PyExc_AttributeError DATA
  PyExc_BaseException=python35.PyExc_BaseException DATA
  PyExc_BufferError=python35.PyExc_BufferError DATA
  PyExc_BytesWarning=python35.PyExc_BytesWarning DATA
  PyExc_DeprecationWarning=python35.PyExc_DeprecationWarning DATA
  PyExc_EOFError=python35.PyExc_EOFError DATA
  PyExc_EnvironmentError=python35.PyExc_EnvironmentError DATA
  PyExc_Exception=python35.PyExc_Exception DATA
  PyExc_FloatingPointError=python35.PyExc_FloatingPointError DATA
  PyExc_FutureWarning=python35.PyExc_FutureWarning DATA
  PyExc_GeneratorExit=python35.PyExc_GeneratorExit DATA
  PyExc_IOError=python35.PyExc_IOError DATA
  PyExc_ImportError=python35.PyExc_ImportError DATA
  PyExc_ImportWarning=python35.PyExc_ImportWarning DATA
  PyExc_IndentationError=python35.PyExc_IndentationError DATA
  PyExc_IndexError=python35.PyExc_IndexError DATA
  PyExc_KeyError=python35.PyExc_KeyError DATA
  PyExc_KeyboardInterrupt=python35.PyExc_KeyboardInterrupt DATA
  PyExc_LookupError=python35.PyExc_LookupError DATA
  PyExc_MemoryError=python35.PyExc_MemoryError DATA
  PyExc_MemoryErrorInst=python35.PyExc_MemoryErrorInst DATA
  PyExc_NameError=python35.PyExc_NameError DATA
  PyExc_NotImplementedError=python35.PyExc_NotImplementedError DATA
  PyExc_OSError=python35.PyExc_OSError DATA
  PyExc_OverflowError=python35.PyExc_OverflowError DATA
  PyExc_PendingDeprecationWarning=python35.PyExc_PendingDeprecationWarning DATA
  PyExc_RecursionErrorInst=python35.PyExc_RecursionErrorInst DATA
  PyExc_ReferenceError=python35.PyExc_ReferenceError DATA
  PyExc_RuntimeError=python35.PyExc_RuntimeError DATA
  PyExc_RuntimeWarning=python35.PyExc_RuntimeWarning DATA
  PyExc_StopIteration=python35.PyExc_StopIteration DATA
  PyExc_SyntaxError=python35.PyExc_SyntaxError DATA
  PyExc_SyntaxWarning=python35.PyExc_SyntaxWarning DATA
  PyExc_SystemError=python35.PyExc_SystemError DATA
  PyExc_SystemExit=python35.PyExc_SystemExit DATA
  PyExc_TabError=python35.PyExc_TabError DATA
  PyExc_TypeError=python35.PyExc_TypeError DATA
  PyExc_UnboundLocalError=python35.PyExc_UnboundLocalError DATA
  PyExc_UnicodeDecodeError=python35.PyExc_UnicodeDecodeError DATA
  PyExc_UnicodeEncodeError=python35.PyExc_UnicodeEncodeError DATA
  PyExc_UnicodeError=python35.PyExc_UnicodeError DATA
  PyExc_UnicodeTranslateError=python35.PyExc_UnicodeTranslateError DATA
  PyExc_UnicodeWarning=python35.PyExc_UnicodeWarning DATA
  PyExc_UserWarning=python35.PyExc_UserWarning DATA
  PyExc_ValueError=python35.PyExc_ValueError DATA
  PyExc_Warning=python35.PyExc_Warning DATA
  PyExc_ZeroDivisionError=python35.PyExc_ZeroDivisionError DATA
  PyException_GetCause=python35.PyException_GetCause
  PyException_GetContext=python35.PyException_GetContext
  PyException_GetTraceback=python35.PyException_GetTraceback
  PyException_SetCause=python35.PyException_SetCause
  PyException_SetContext=python35.PyException_SetContext
  PyException_SetTraceback=python35.PyException_SetTraceback
  PyFile_FromFd=python35.PyFile_FromFd
  PyFile_GetLine=python35.PyFile_GetLine
  PyFile_WriteObject=python35.PyFile_WriteObject
  PyFile_WriteString=python35.PyFile_WriteString
  PyFilter_Type=python35.PyFilter_Type DATA
  PyFloat_AsDouble=python35.PyFloat_AsDouble
  PyFloat_FromDouble=python35.PyFloat_FromDouble
  PyFloat_FromString=python35.PyFloat_FromString
  PyFloat_GetInfo=python35.PyFloat_GetInfo
  PyFloat_GetMax=python35.PyFloat_GetMax
  PyFloat_GetMin=python35.PyFloat_GetMin
  PyFloat_Type=python35.PyFloat_Type DATA
  PyFrozenSet_New=python35.PyFrozenSet_New
  PyFrozenSet_Type=python35.PyFrozenSet_Type DATA
  PyGC_Collect=python35.PyGC_Collect
  PyGILState_Ensure=python35.PyGILState_Ensure
  PyGILState_GetThisThreadState=python35.PyGILState_GetThisThreadState
  PyGILState_Release=python35.PyGILState_Release
  PyGetSetDescr_Type=python35.PyGetSetDescr_Type DATA
  PyImport_AddModule=python35.PyImport_AddModule
  PyImport_AppendInittab=python35.PyImport_AppendInittab
  PyImport_Cleanup=python35.PyImport_Cleanup
  PyImport_ExecCodeModule=python35.PyImport_ExecCodeModule
  PyImport_ExecCodeModuleEx=python35.PyImport_ExecCodeModuleEx
  PyImport_ExecCodeModuleWithPathnames=python35.PyImport_ExecCodeModuleWithPathnames
  PyImport_GetImporter=python35.PyImport_GetImporter
  PyImport_GetMagicNumber=python35.PyImport_GetMagicNumber
  PyImport_GetMagicTag=python35.PyImport_GetMagicTag
  PyImport_GetModuleDict=python35.PyImport_GetModuleDict
  PyImport_Import=python35.PyImport_Import
  PyImport_ImportFrozenModule=python35.PyImport_ImportFrozenModule
  PyImport_ImportModule=python35.PyImport_ImportModule
  PyImport_ImportModuleLevel=python35.PyImport_ImportModuleLevel
  PyImport_ImportModuleNoBlock=python35.PyImport_ImportModuleNoBlock
  PyImport_ReloadModule=python35.PyImport_ReloadModule
  PyInterpreterState_Clear=python35.PyInterpreterState_Clear
  PyInterpreterState_Delete=python35.PyInterpreterState_Delete
  PyInterpreterState_New=python35.PyInterpreterState_New
  PyIter_Next=python35.PyIter_Next
  PyListIter_Type=python35.PyListIter_Type DATA
  PyListRevIter_Type=python35.PyListRevIter_Type DATA
  PyList_Append=python35.PyList_Append
  PyList_AsTuple=python35.PyList_AsTuple
  PyList_GetItem=python35.PyList_GetItem
  PyList_GetSlice=python35.PyList_GetSlice
  PyList_Insert=python35.PyList_Insert
  PyList_New=python35.PyList_New
  PyList_Reverse=python35.PyList_Reverse
  PyList_SetItem=python35.PyList_SetItem
  PyList_SetSlice=python35.PyList_SetSlice
  PyList_Size=python35.PyList_Size
  PyList_Sort=python35.PyList_Sort
  PyList_Type=python35.PyList_Type DATA
  PyLongRangeIter_Type=python35.PyLongRangeIter_Type DATA
  PyLong_AsDouble=python35.PyLong_AsDouble
  PyLong_AsLong=python35.PyLong_AsLong
  PyLong_AsLongAndOverflow=python35.PyLong_AsLongAndOverflow
  PyLong_AsLongLong=python35.PyLong_AsLongLong
  PyLong_AsLongLongAndOverflow=python35.PyLong_AsLongLongAndOverflow
  PyLong_AsSize_t=python35.PyLong_AsSize_t
  PyLong_AsSsize_t=python35.PyLong_AsSsize_t
  PyLong_AsUnsignedLong=python35.PyLong_AsUnsignedLong
  PyLong_AsUnsignedLongLong=python35.PyLong_AsUnsignedLongLong
  PyLong_AsUnsignedLongLongMask=python35.PyLong_AsUnsignedLongLongMask
  PyLong_AsUnsignedLongMask=python35.PyLong_AsUnsignedLongMask
  PyLong_AsVoidPtr=python35.PyLong_AsVoidPtr
  PyLong_FromDouble=python35.PyLong_FromDouble
  PyLong_FromLong=python35.PyLong_FromLong
  PyLong_FromLongLong=python35.PyLong_FromLongLong
  PyLong_FromSize_t=python35.PyLong_FromSize_t
  PyLong_FromSsize_t=python35.PyLong_FromSsize_t
  PyLong_FromString=python35.PyLong_FromString
  PyLong_FromUnsignedLong=python35.PyLong_FromUnsignedLong
  PyLong_FromUnsignedLongLong=python35.PyLong_FromUnsignedLongLong
  PyLong_FromVoidPtr=python35.PyLong_FromVoidPtr
  PyLong_GetInfo=python35.PyLong_GetInfo
  PyLong_Type=python35.PyLong_Type DATA
  PyMap_Type=python35.PyMap_Type DATA
  PyMapping_Check=python35.PyMapping_Check
  PyMapping_GetItemString=python35.PyMapping_GetItemString
  PyMapping_HasKey=python35.PyMapping_HasKey
  PyMapping_HasKeyString=python35.PyMapping_HasKeyString
  PyMapping_Items=python35.PyMapping_Items
  PyMapping_Keys=python35.PyMapping_Keys
  PyMapping_Length=python35.PyMapping_Length
  PyMapping_SetItemString=python35.PyMapping_SetItemString
  PyMapping_Size=python35.PyMapping_Size
  PyMapping_Values=python35.PyMapping_Values
  PyMem_Free=python35.PyMem_Free
  PyMem_Malloc=python35.PyMem_Malloc
  PyMem_Realloc=python35.PyMem_Realloc
  PyMemberDescr_Type=python35.PyMemberDescr_Type DATA
  PyMemoryView_FromObject=python35.PyMemoryView_FromObject
  PyMemoryView_GetContiguous=python35.PyMemoryView_GetContiguous
  PyMemoryView_Type=python35.PyMemoryView_Type DATA
  PyMethodDescr_Type=python35.PyMethodDescr_Type DATA
  PyModule_AddIntConstant=python35.PyModule_AddIntConstant
  PyModule_AddObject=python35.PyModule_AddObject
  PyModule_AddStringConstant=python35.PyModule_AddStringConstant
  PyModule_Create2=python35.PyModule_Create2
  PyModule_GetDef=python35.PyModule_GetDef
  PyModule_GetDict=python35.PyModule_GetDict
  PyModule_GetFilename=python35.PyModule_GetFilename
  PyModule_GetFilenameObject=python35.PyModule_GetFilenameObject
  PyModule_GetName=python35.PyModule_GetName
  PyModule_GetState=python35.PyModule_GetState
  PyModule_New=python35.PyModule_New
  PyModule_Type=python35.PyModule_Type DATA
  PyModuleDef_Init=python35.PyModuleDef_Init
  PyModuleDef_Type=python35.PyModuleDef_Type DATA
  PyNullImporter_Type=python35.PyNullImporter_Type DATA
  PyNumber_Absolute=python35.PyNumber_Absolute
  PyNumber_Add=python35.PyNumber_Add
  PyNumber_And=python35.PyNumber_And
  PyNumber_AsSsize_t=python35.PyNumber_AsSsize_t
  PyNumber_Check=python35.PyNumber_Check
  PyNumber_Divmod=python35.PyNumber_Divmod
  PyNumber_Float=python35.PyNumber_Float
  PyNumber_FloorDivide=python35.PyNumber_FloorDivide
  PyNumber_InPlaceAdd=python35.PyNumber_InPlaceAdd
  PyNumber_InPlaceAnd=python35.PyNumber_InPlaceAnd
  PyNumber_InPlaceFloorDivide=python35.PyNumber_InPlaceFloorDivide
  PyNumber_InPlaceLshift=python35.PyNumber_InPlaceLshift
  PyNumber_InPlaceMultiply=python35.PyNumber_InPlaceMultiply
  PyNumber_InPlaceOr=python35.PyNumber_InPlaceOr
  PyNumber_InPlacePower=python35.PyNumber_InPlacePower
  PyNumber_InPlaceRemainder=python35.PyNumber_InPlaceRemainder
  PyNumber_InPlaceRshift=python35.PyNumber_InPlaceRshift
  PyNumber_InPlaceSubtract=python35.PyNumber_InPlaceSubtract
  PyNumber_InPlaceTrueDivide=python35.PyNumber_InPlaceTrueDivide
  PyNumber_InPlaceXor=python35.PyNumber_InPlaceXor
  PyNumber_Index=python35.PyNumber_Index
  PyNumber_Invert=python35.PyNumber_Invert
  PyNumber_Long=python35.PyNumber_Long
  PyNumber_Lshift=python35.PyNumber_Lshift
  PyNumber_Multiply=python35.PyNumber_Multiply
  PyNumber_Negative=python35.PyNumber_Negative
  PyNumber_Or=python35.PyNumber_Or
  PyNumber_Positive=python35.PyNumber_Positive
  PyNumber_Power=python35.PyNumber_Power
  PyNumber_Remainder=python35.PyNumber_Remainder
  PyNumber_Rshift=python35.PyNumber_Rshift
  PyNumber_Subtract=python35.PyNumber_Subtract
  PyNumber_ToBase=python35.PyNumber_ToBase
  PyNumber_TrueDivide=python35.PyNumber_TrueDivide
  PyNumber_Xor=python35.PyNumber_Xor
  PyOS_AfterFork=python35.PyOS_AfterFork
  PyOS_InitInterrupts=python35.PyOS_InitInterrupts
  PyOS_InputHook=python35.PyOS_InputHook DATA
  PyOS_InterruptOccurred=python35.PyOS_InterruptOccurred
  PyOS_ReadlineFunctionPointer=python35.PyOS_ReadlineFunctionPointer DATA
  PyOS_double_to_string=python35.PyOS_double_to_string
  PyOS_getsig=python35.PyOS_getsig
  PyOS_mystricmp=python35.PyOS_mystricmp
  PyOS_mystrnicmp=python35.PyOS_mystrnicmp
  PyOS_setsig=python35.PyOS_setsig
  PyOS_snprintf=python35.PyOS_snprintf
  PyOS_string_to_double=python35.PyOS_string_to_double
  PyOS_strtol=python35.PyOS_strtol
  PyOS_strtoul=python35.PyOS_strtoul
  PyOS_vsnprintf=python35.PyOS_vsnprintf
  PyObject_ASCII=python35.PyObject_ASCII
  PyObject_AsCharBuffer=python35.PyObject_AsCharBuffer
  PyObject_AsFileDescriptor=python35.PyObject_AsFileDescriptor
  PyObject_AsReadBuffer=python35.PyObject_AsReadBuffer
  PyObject_AsWriteBuffer=python35.PyObject_AsWriteBuffer
  PyObject_Bytes=python35.PyObject_Bytes
  PyObject_Call=python35.PyObject_Call
  PyObject_CallFunction=python35.PyObject_CallFunction
  PyObject_CallFunctionObjArgs=python35.PyObject_CallFunctionObjArgs
  PyObject_CallMethod=python35.PyObject_CallMethod
  PyObject_CallMethodObjArgs=python35.PyObject_CallMethodObjArgs
  PyObject_CallObject=python35.PyObject_CallObject
  PyObject_CheckReadBuffer=python35.PyObject_CheckReadBuffer
  PyObject_ClearWeakRefs=python35.PyObject_ClearWeakRefs
  PyObject_DelItem=python35.PyObject_DelItem
  PyObject_DelItemString=python35.PyObject_DelItemString
  PyObject_Dir=python35.PyObject_Dir
  PyObject_Format=python35.PyObject_Format
  PyObject_Free=python35.PyObject_Free
  PyObject_GC_Del=python35.PyObject_GC_Del
  PyObject_GC_Track=python35.PyObject_GC_Track
  PyObject_GC_UnTrack=python35.PyObject_GC_UnTrack
  PyObject_GenericGetAttr=python35.PyObject_GenericGetAttr
  PyObject_GenericSetAttr=python35.PyObject_GenericSetAttr
  PyObject_GetAttr=python35.PyObject_GetAttr
  PyObject_GetAttrString=python35.PyObject_GetAttrString
  PyObject_GetItem=python35.PyObject_GetItem
  PyObject_GetIter=python35.PyObject_GetIter
  PyObject_HasAttr=python35.PyObject_HasAttr
  PyObject_HasAttrString=python35.PyObject_HasAttrString
  PyObject_Hash=python35.PyObject_Hash
  PyObject_HashNotImplemented=python35.PyObject_HashNotImplemented
  PyObject_Init=python35.PyObject_Init
  PyObject_InitVar=python35.PyObject_InitVar
  PyObject_IsInstance=python35.PyObject_IsInstance
  PyObject_IsSubclass=python35.PyObject_IsSubclass
  PyObject_IsTrue=python35.PyObject_IsTrue
  PyObject_Length=python35.PyObject_Length
  PyObject_Malloc=python35.PyObject_Malloc
  PyObject_Not=python35.PyObject_Not
  PyObject_Realloc=python35.PyObject_Realloc
  PyObject_Repr=python35.PyObject_Repr
  PyObject_RichCompare=python35.PyObject_RichCompare
  PyObject_RichCompareBool=python35.PyObject_RichCompareBool
  PyObject_SelfIter=python35.PyObject_SelfIter
  PyObject_SetAttr=python35.PyObject_SetAttr
  PyObject_SetAttrString=python35.PyObject_SetAttrString
  PyObject_SetItem=python35.PyObject_SetItem
  PyObject_Size=python35.PyObject_Size
  PyObject_Str=python35.PyObject_Str
  PyObject_Type=python35.PyObject_Type DATA
  PyODict_DelItem=python35.PyODict_DelItem
  PyODict_New=python35.PyODict_New
  PyODict_SetItem=python35.PyODict_SetItem
  PyODict_Type=python35.PyODict_Type DATA
  PyODictItems_Type=python35.PyODictItems_Type DATA
  PyODictIter_Type=python35.PyODictIter_Type DATA
  PyODictKeys_Type=python35.PyODictKeys_Type DATA
  PyODictValues_Type=python35.PyODictValues_Type DATA
  PyParser_SimpleParseFileFlags=python35.PyParser_SimpleParseFileFlags
  PyParser_SimpleParseStringFlags=python35.PyParser_SimpleParseStringFlags
  PyProperty_Type=python35.PyProperty_Type DATA
  PyRangeIter_Type=python35.PyRangeIter_Type DATA
  PyRange_Type=python35.PyRange_Type DATA
  PyReversed_Type=python35.PyReversed_Type DATA
  PySeqIter_New=python35.PySeqIter_New
  PySeqIter_Type=python35.PySeqIter_Type DATA
  PySequence_Check=python35.PySequence_Check
  PySequence_Concat=python35.PySequence_Concat
  PySequence_Contains=python35.PySequence_Contains
  PySequence_Count=python35.PySequence_Count
  PySequence_DelItem=python35.PySequence_DelItem
  PySequence_DelSlice=python35.PySequence_DelSlice
  PySequence_Fast=python35.PySequence_Fast
  PySequence_GetItem=python35.PySequence_GetItem
  PySequence_GetSlice=python35.PySequence_GetSlice
  PySequence_In=python35.PySequence_In
  PySequence_InPlaceConcat=python35.PySequence_InPlaceConcat
  PySequence_InPlaceRepeat=python35.PySequence_InPlaceRepeat
  PySequence_Index=python35.PySequence_Index
  PySequence_Length=python35.PySequence_Length
  PySequence_List=python35.PySequence_List
  PySequence_Repeat=python35.PySequence_Repeat
  PySequence_SetItem=python35.PySequence_SetItem
  PySequence_SetSlice=python35.PySequence_SetSlice
  PySequence_Size=python35.PySequence_Size
  PySequence_Tuple=python35.PySequence_Tuple
  PySetIter_Type=python35.PySetIter_Type DATA
  PySet_Add=python35.PySet_Add
  PySet_Clear=python35.PySet_Clear
  PySet_Contains=python35.PySet_Contains
  PySet_Discard=python35.PySet_Discard
  PySet_New=python35.PySet_New
  PySet_Pop=python35.PySet_Pop
  PySet_Size=python35.PySet_Size
  PySet_Type=python35.PySet_Type DATA
  PySlice_GetIndices=python35.PySlice_GetIndices
  PySlice_GetIndicesEx=python35.PySlice_GetIndicesEx
  PySlice_New=python35.PySlice_New
  PySlice_Type=python35.PySlice_Type DATA
  PySortWrapper_Type=python35.PySortWrapper_Type DATA
  PyState_FindModule=python35.PyState_FindModule
  PyState_AddModule=python35.PyState_AddModule
  PyState_RemoveModule=python35.PyState_RemoveModule
  PyStructSequence_GetItem=python35.PyStructSequence_GetItem
  PyStructSequence_New=python35.PyStructSequence_New
  PyStructSequence_NewType=python35.PyStructSequence_NewType
  PyStructSequence_SetItem=python35.PyStructSequence_SetItem
  PySuper_Type=python35.PySuper_Type DATA
  PySys_AddWarnOption=python35.PySys_AddWarnOption
  PySys_AddWarnOptionUnicode=python35.PySys_AddWarnOptionUnicode
  PySys_FormatStderr=python35.PySys_FormatStderr
  PySys_FormatStdout=python35.PySys_FormatStdout
  PySys_GetObject=python35.PySys_GetObject
  PySys_HasWarnOptions=python35.PySys_HasWarnOptions
  PySys_ResetWarnOptions=python35.PySys_ResetWarnOptions
  PySys_SetArgv=python35.PySys_SetArgv
  PySys_SetArgvEx=python35.PySys_SetArgvEx
  PySys_SetObject=python35.PySys_SetObject
  PySys_SetPath=python35.PySys_SetPath
  PySys_WriteStderr=python35.PySys_WriteStderr
  PySys_WriteStdout=python35.PySys_WriteStdout
  PyThreadState_Clear=python35.PyThreadState_Clear
  PyThreadState_Delete=python35.PyThreadState_Delete
  PyThreadState_DeleteCurrent=python35.PyThreadState_DeleteCurrent
  PyThreadState_Get=python35.PyThreadState_Get
  PyThreadState_GetDict=python35.PyThreadState_GetDict
  PyThreadState_New=python35.PyThreadState_New
  PyThreadState_SetAsyncExc=python35.PyThreadState_SetAsyncExc
  PyThreadState_Swap=python35.PyThreadState_Swap
  PyTraceBack_Here=python35.PyTraceBack_Here
  PyTraceBack_Print=python35.PyTraceBack_Print
  PyTraceBack_Type=python35.PyTraceBack_Type DATA
  PyTupleIter_Type=python35.PyTupleIter_Type DATA
  PyTuple_ClearFreeList=python35.PyTuple_ClearFreeList
  PyTuple_GetItem=python35.PyTuple_GetItem
  PyTuple_GetSlice=python35.PyTuple_GetSlice
  PyTuple_New=python35.PyTuple_New
  PyTuple_Pack=python35.PyTuple_Pack
  PyTuple_SetItem=python35.PyTuple_SetItem
  PyTuple_Size=python35.PyTuple_Size
  PyTuple_Type=python35.PyTuple_Type DATA
  PyType_ClearCache=python35.PyType_ClearCache
  PyType_FromSpec=python35.PyType_FromSpec
  PyType_FromSpecWithBases=python35.PyType_FromSpecWithBases
  PyType_GenericAlloc=python35.PyType_GenericAlloc
  PyType_GenericNew=python35.PyType_GenericNew
  PyType_GetFlags=python35.PyType_GetFlags
  PyType_GetSlot=python35.PyType_GetSlot
  PyType_IsSubtype=python35.PyType_IsSubtype
  PyType_Modified=python35.PyType_Modified
  PyType_Ready=python35.PyType_Ready
  PyType_Type=python35.PyType_Type DATA
  PyUnicodeDecodeError_Create=python35.PyUnicodeDecodeError_Create
  PyUnicodeDecodeError_GetEncoding=python35.PyUnicodeDecodeError_GetEncoding
  PyUnicodeDecodeError_GetEnd=python35.PyUnicodeDecodeError_GetEnd
  PyUnicodeDecodeError_GetObject=python35.PyUnicodeDecodeError_GetObject
  PyUnicodeDecodeError_GetReason=python35.PyUnicodeDecodeError_GetReason
  PyUnicodeDecodeError_GetStart=python35.PyUnicodeDecodeError_GetStart
  PyUnicodeDecodeError_SetEnd=python35.PyUnicodeDecodeError_SetEnd
  PyUnicodeDecodeError_SetReason=python35.PyUnicodeDecodeError_SetReason
  PyUnicodeDecodeError_SetStart=python35.PyUnicodeDecodeError_SetStart
  PyUnicodeEncodeError_GetEncoding=python35.PyUnicodeEncodeError_GetEncoding
  PyUnicodeEncodeError_GetEnd=python35.PyUnicodeEncodeError_GetEnd
  PyUnicodeEncodeError_GetObject=python35.PyUnicodeEncodeError_GetObject
  PyUnicodeEncodeError_GetReason=python35.PyUnicodeEncodeError_GetReason
  PyUnicodeEncodeError_GetStart=python35.PyUnicodeEncodeError_GetStart
  PyUnicodeEncodeError_SetEnd=python35.PyUnicodeEncodeError_SetEnd
  PyUnicodeEncodeError_SetReason=python35.PyUnicodeEncodeError_SetReason
  PyUnicodeEncodeError_SetStart=python35.PyUnicodeEncodeError_SetStart
  PyUnicodeIter_Type=python35.PyUnicodeIter_Type DATA
  PyUnicodeTranslateError_GetEnd=python35.PyUnicodeTranslateError_GetEnd
  PyUnicodeTranslateError_GetObject=python35.PyUnicodeTranslateError_GetObject
  PyUnicodeTranslateError_GetReason=python35.PyUnicodeTranslateError_GetReason
  PyUnicodeTranslateError_GetStart=python35.PyUnicodeTranslateError_GetStart
  PyUnicodeTranslateError_SetEnd=python35.PyUnicodeTranslateError_SetEnd
  PyUnicodeTranslateError_SetReason=python35.PyUnicodeTranslateError_SetReason
  PyUnicodeTranslateError_SetStart=python35.PyUnicodeTranslateError_SetStart
  PyUnicode_Append=python35.PyUnicode_Append
  PyUnicode_AppendAndDel=python35.PyUnicode_AppendAndDel
  PyUnicode_AsASCIIString=python35.PyUnicode_AsASCIIString
  PyUnicode_AsCharmapString=python35.PyUnicode_AsCharmapString
  PyUnicode_AsDecodedObject=python35.PyUnicode_AsDecodedObject
  PyUnicode_AsDecodedUnicode=python35.PyUnicode_AsDecodedUnicode
  PyUnicode_AsEncodedObject=python35.PyUnicode_AsEncodedObject
  PyUnicode_AsEncodedString=python35.PyUnicode_AsEncodedString
  PyUnicode_AsEncodedUnicode=python35.PyUnicode_AsEncodedUnicode
  PyUnicode_AsLatin1String=python35.PyUnicode_AsLatin1String
  PyUnicode_AsRawUnicodeEscapeString=python35.PyUnicode_AsRawUnicodeEscapeString
  PyUnicode_AsUTF16String=python35.PyUnicode_AsUTF16String
  PyUnicode_AsUTF32String=python35.PyUnicode_AsUTF32String
  PyUnicode_AsUTF8String=python35.PyUnicode_AsUTF8String
  PyUnicode_AsUnicodeEscapeString=python35.PyUnicode_AsUnicodeEscapeString
  PyUnicode_AsWideChar=python35.PyUnicode_AsWideChar
  PyUnicode_ClearFreelist=python35.PyUnicode_ClearFreelist
  PyUnicode_Compare=python35.PyUnicode_Compare
  PyUnicode_Concat=python35.PyUnicode_Concat
  PyUnicode_Contains=python35.PyUnicode_Contains
  PyUnicode_Count=python35.PyUnicode_Count
  PyUnicode_Decode=python35.PyUnicode_Decode
  PyUnicode_DecodeASCII=python35.PyUnicode_DecodeASCII
  PyUnicode_DecodeCharmap=python35.PyUnicode_DecodeCharmap
  PyUnicode_DecodeFSDefault=python35.PyUnicode_DecodeFSDefault
  PyUnicode_DecodeFSDefaultAndSize=python35.PyUnicode_DecodeFSDefaultAndSize
  PyUnicode_DecodeLatin1=python35.PyUnicode_DecodeLatin1
  PyUnicode_DecodeRawUnicodeEscape=python35.PyUnicode_DecodeRawUnicodeEscape
  PyUnicode_DecodeUTF16=python35.PyUnicode_DecodeUTF16
  PyUnicode_DecodeUTF16Stateful=python35.PyUnicode_DecodeUTF16Stateful
  PyUnicode_DecodeUTF32=python35.PyUnicode_DecodeUTF32
  PyUnicode_DecodeUTF32Stateful=python35.PyUnicode_DecodeUTF32Stateful
  PyUnicode_DecodeUTF8=python35.PyUnicode_DecodeUTF8
  PyUnicode_DecodeUTF8Stateful=python35.PyUnicode_DecodeUTF8Stateful
  PyUnicode_DecodeUnicodeEscape=python35.PyUnicode_DecodeUnicodeEscape
  PyUnicode_FSConverter=python35.PyUnicode_FSConverter
  PyUnicode_FSDecoder=python35.PyUnicode_FSDecoder
  PyUnicode_Find=python35.PyUnicode_Find
  PyUnicode_Format=python35.PyUnicode_Format
  PyUnicode_FromEncodedObject=python35.PyUnicode_FromEncodedObject
  PyUnicode_FromFormat=python35.PyUnicode_FromFormat
  PyUnicode_FromFormatV=python35.PyUnicode_FromFormatV
  PyUnicode_FromObject=python35.PyUnicode_FromObject
  PyUnicode_FromOrdinal=python35.PyUnicode_FromOrdinal
  PyUnicode_FromString=python35.PyUnicode_FromString
  PyUnicode_FromStringAndSize=python35.PyUnicode_FromStringAndSize
  PyUnicode_FromWideChar=python35.PyUnicode_FromWideChar
  PyUnicode_GetDefaultEncoding=python35.PyUnicode_GetDefaultEncoding
  PyUnicode_GetSize=python35.PyUnicode_GetSize
  PyUnicode_IsIdentifier=python35.PyUnicode_IsIdentifier
  PyUnicode_Join=python35.PyUnicode_Join
  PyUnicode_Partition=python35.PyUnicode_Partition
  PyUnicode_RPartition=python35.PyUnicode_RPartition
  PyUnicode_RSplit=python35.PyUnicode_RSplit
  PyUnicode_Replace=python35.PyUnicode_Replace
  PyUnicode_Resize=python35.PyUnicode_Resize
  PyUnicode_RichCompare=python35.PyUnicode_RichCompare
  PyUnicode_SetDefaultEncoding=python35.PyUnicode_SetDefaultEncoding
  PyUnicode_Split=python35.PyUnicode_Split
  PyUnicode_Splitlines=python35.PyUnicode_Splitlines
  PyUnicode_Tailmatch=python35.PyUnicode_Tailmatch
  PyUnicode_Translate=python35.PyUnicode_Translate
  PyUnicode_BuildEncodingMap=python35.PyUnicode_BuildEncodingMap
  PyUnicode_CompareWithASCIIString=python35.PyUnicode_CompareWithASCIIString
  PyUnicode_DecodeUTF7=python35.PyUnicode_DecodeUTF7
  PyUnicode_DecodeUTF7Stateful=python35.PyUnicode_DecodeUTF7Stateful
  PyUnicode_EncodeFSDefault=python35.PyUnicode_EncodeFSDefault
  PyUnicode_InternFromString=python35.PyUnicode_InternFromString
  PyUnicode_InternImmortal=python35.PyUnicode_InternImmortal
  PyUnicode_InternInPlace=python35.PyUnicode_InternInPlace
  PyUnicode_Type=python35.PyUnicode_Type DATA
  PyWeakref_GetObject=python35.PyWeakref_GetObject DATA
  PyWeakref_NewProxy=python35.PyWeakref_NewProxy
  PyWeakref_NewRef=python35.PyWeakref_NewRef
  PyWrapperDescr_Type=python35.PyWrapperDescr_Type DATA
  PyWrapper_New=python35.PyWrapper_New
  PyZip_Type=python35.PyZip_Type DATA
  Py_AddPendingCall=python35.Py_AddPendingCall
  Py_AtExit=python35.Py_AtExit
  Py_BuildValue=python35.Py_BuildValue
  Py_CompileString=python35.Py_CompileString
  Py_DecRef=python35.Py_DecRef
  Py_EndInterpreter=python35.Py_EndInterpreter
  Py_Exit=python35.Py_Exit
  Py_FatalError=python35.Py_FatalError
  Py_FileSystemDefaultEncoding=python35.Py_FileSystemDefaultEncoding DATA
  Py_Finalize=python35.Py_Finalize
  Py_GetBuildInfo=python35.Py_GetBuildInfo
  Py_GetCompiler=python35.Py_GetCompiler
  Py_GetCopyright=python35.Py_GetCopyright
  Py_GetExecPrefix=python35.Py_GetExecPrefix
  Py_GetPath=python35.Py_GetPath
  Py_GetPlatform=python35.Py_GetPlatform
  Py_GetPrefix=python35.Py_GetPrefix
  Py_GetProgramFullPath=python35.Py_GetProgramFullPath
  Py_GetProgramName=python35.Py_GetProgramName
  Py_GetPythonHome=python35.Py_GetPythonHome
  Py_GetRecursionLimit=python35.Py_GetRecursionLimit
  Py_GetVersion=python35.Py_GetVersion
  Py_HasFileSystemDefaultEncoding=python35.Py_HasFileSystemDefaultEncoding DATA
  Py_IncRef=python35.Py_IncRef
  Py_Initialize=python35.Py_Initialize
  Py_InitializeEx=python35.Py_InitializeEx
  Py_IsInitialized=python35.Py_IsInitialized
  Py_Main=python35.Py_Main
  Py_MakePendingCalls=python35.Py_MakePendingCalls
  Py_NewInterpreter=python35.Py_NewInterpreter
  Py_ReprEnter=python35.Py_ReprEnter
  Py_ReprLeave=python35.Py_ReprLeave
  Py_SetProgramName=python35.Py_SetProgramName
  Py_SetPythonHome=python35.Py_SetPythonHome
  Py_SetRecursionLimit=python35.Py_SetRecursionLimit
  Py_SymtableString=python35.Py_SymtableString
  Py_VaBuildValue=python35.Py_VaBuildValue
  _PyErr_BadInternalCall=python35._PyErr_BadInternalCall
  _PyObject_CallFunction_SizeT=python35._PyObject_CallFunction_SizeT
  _PyObject_CallMethod_SizeT=python35._PyObject_CallMethod_SizeT
  _PyObject_GC_Malloc=python35._PyObject_GC_Malloc
  _PyObject_GC_New=python35._PyObject_GC_New
  _PyObject_GC_NewVar=python35._PyObject_GC_NewVar
  _PyObject_GC_Resize=python35._PyObject_GC_Resize
  _PyObject_New=python35._PyObject_New
  _PyObject_NewVar=python35._PyObject_NewVar
  _PyState_AddModule=python35._PyState_AddModule
  _PyThreadState_Init=python35._PyThreadState_Init
  _PyThreadState_Prealloc=python35._PyThreadState_Prealloc
  _PyTrash_delete_later=python35._PyTrash_delete_later DATA
  _PyTrash_delete_nesting=python35._PyTrash_delete_nesting DATA
  _PyTrash_deposit_object=python35._PyTrash_deposit_object
  _PyTrash_destroy_chain=python35._PyTrash_destroy_chain
  _PyWeakref_CallableProxyType=python35._PyWeakref_CallableProxyType DATA
  _PyWeakref_ProxyType=python35._PyWeakref_ProxyType DATA
  _PyWeakref_RefType=python35._PyWeakref_RefType DATA
  _Py_BuildValue_SizeT=python35._Py_BuildValue_SizeT
  _Py_CheckRecursionLimit=python35._Py_CheckRecursionLimit DATA
  _Py_CheckRecursiveCall=python35._Py_CheckRecursiveCall
  _Py_Dealloc=python35._Py_Dealloc
  _Py_EllipsisObject=python35._Py_EllipsisObject DATA
  _Py_FalseStruct=python35._Py_FalseStruct DATA
  _Py_NoneStruct=python35._Py_NoneStruct DATA
  _Py_NotImplementedStruct=python35._Py_NotImplementedStruct DATA
  _Py_SwappedOp=python35._Py_SwappedOp DATA
  _Py_TrueStruct=python35._Py_TrueStruct DATA
  _Py_VaBuildValue_SizeT=python35._Py_VaBuildValue_SizeT
  _PyArg_Parse_SizeT=python35._PyArg_Parse_SizeT
  _PyArg_ParseTuple_SizeT=python35._PyArg_ParseTuple_SizeT
  _PyArg_ParseTupleAndKeywords_SizeT=python35._PyArg_ParseTupleAndKeywords_SizeT
  _PyArg_VaParse_SizeT=python35._PyArg_VaParse_SizeT
  _PyArg_VaParseTupleAndKeywords_SizeT=python35._PyArg_VaParseTupleAndKeywords_SizeT
  _Py_BuildValue_SizeT=python35._Py_BuildValue_SizeT
